use std::{
    collections::{BTreeMap, HashMap},
    fs,
};

use actix_multipart::form::{tempfile::TempFile, text::Text, MultipartForm};
use actix_web::{post, web, HttpResponse};
use color_eyre::{eyre::eyre, Result};
use lettre::{
    message::{header::ContentType, Attachment, MultiPart, SinglePart},
    Message,
};
use maud::{html, Markup, PreEscaped, DOCTYPE};
use reqwest::Client;
use serde_json::json;
use sqlx::SqliteConnection;
use tracing::{error, info};

#[derive(Debug, MultipartForm)]
struct MtParentForm {
    #[multipart(rename = "firstname")]
    first_name: Text<String>,
    #[multipart(rename = "lastname")]
    last_name: Text<String>,
    email: Text<String>,
    #[multipart(rename = "studentfirstname")]
    student_first_name: Text<String>,
    #[multipart(rename = "studentlastname")]
    student_last_name: Text<String>,
    authority: Text<String>,
    positive: Text<String>,
    negative: Text<String>,
    #[multipart(rename = "family-relation")]
    family: Text<String>,
    #[multipart(rename = "previous-trip-info")]
    previous_trip: Text<String>,
    #[multipart(rename = "trip-feelings")]
    feelings: Text<String>,
    #[multipart(rename = "extra-info")]
    extra_info: Text<String>,
}

impl From<&MtParentForm> for HashMap<i32, String> {
    fn from(form: &MtParentForm) -> Self {
        let mut map = HashMap::new();
        map.insert(167, format!("{} {}", form.first_name.0, form.last_name.0));
        map.insert(
            168,
            format!("{} {}", form.student_first_name.0, form.student_last_name.0),
        );
        map.insert(169, form.authority.0.clone());
        map.insert(170, form.positive.0.clone());
        map.insert(171, form.negative.0.clone());
        map.insert(172, form.family.0.clone());
        map.insert(173, form.previous_trip.0.clone());
        map.insert(174, form.feelings.0.clone());
        map.insert(175, form.extra_info.0.clone());
        map.insert(176, form.email.0.clone());
        map
    }
}

impl MtParentForm {
    async fn build_email(&self) -> Markup {
        html! {
            (DOCTYPE)
                meta charset="utf-8";
                html {
                    head {
                        title { (self.first_name.0) " " (self.last_name.0) " filled out a parent form for " (self.student_first_name.0) " " (self.student_last_name.0) "!" }
                        style {
                            "table { border-collapse: collapse; width: 100% }"
                                "td, th { padding: 8px }"
                                "td { text-align: left; width: 70%; word-wrap: break-word }"
                                "th { text-align: right; border-right: 1px solid #ddd }"
                                "tr { border-bottom: 1px solid #ddd }"
                                "h1 { text-align: center }"
                        }
                    }
                    body {
                        h1 { "Parent reference form for " (self.student_first_name.0) " " (self.student_last_name.0) "!" }
                        hr;
                        table {
                            tr {
                                th { "Name" }
                                td { (self.first_name.0) " " (self.last_name.0) }
                            }
                            tr {
                                th { "Student" }
                                td { (self.student_first_name.0) " " (self.student_last_name.0) }
                            }
                            tr {
                                th { "Email" }
                                td { (self.email.0) }
                            }
                            tr {
                                th { "Authority" }
                                td { (self.authority.0) }
                            }
                            tr {
                                th { "Positive characteristics" }
                                td { (self.positive.0) }
                            }
                            tr {
                                th { "Negative characteristics" }
                                td { (self.negative.0) }
                            }
                            tr {
                                th { "Family Relations" }
                                td { (self.family.0) }
                            }
                            tr {
                                th { "Previous Trip" }
                                td { (self.previous_trip.0) }
                            }
                            tr {
                                th { "Trip Feelings" }
                                td { (self.feelings.0) }
                            }
                            tr {
                                th { "Other Relevant Info" }
                                td { (self.extra_info.0) }
                            }
                        }
                    }
                }
        }
    }

    async fn store_form(&self) -> Result<()> {
        let client = Client::new();
        let map = HashMap::from(self);
        let mut json = HashMap::new();
        json.insert("data", map);

        let link = r#"https://staff.tfcconnection.org/apps/tables/#/table/14/row/757"#;
        let res = client
            .post("https://staff.tfcconnection.org/ocs/v2.php/apps/tables/api/2/tables/14/rows")
            .basic_auth("chris", Some("2VHeGxeC^Zf9KqFK^G@Pt!zu2q^6@b"))
            .header("OCS-APIRequest", "true")
            .header("Content-Type", "application/json")
            .json(&json)
            .send()
            .await?;
        if res.status().is_success() {
            let res = res.text().await.unwrap();
            Ok(())
        } else {
            Err(eyre!(
                "Problem in storing data: {:?}",
                res.error_for_status()
            ))
        }
    }

    async fn send_email(&mut self) -> Result<()> {
        let first = self.student_first_name.clone();
        let last = self.student_last_name.clone();
        let email_subject = format!("Parent reference form for {} {}!", first, last);
        info!("{first} {last} parent reference form!");
        let email = self.build_email().await;
        let email = SinglePart::html(email.into_string());

        if let Ok(m) = Message::builder()
            .from(
                "TFC ADMIN <no-reply@mail.tfcconnection.org>"
                    .parse()
                    .unwrap(),
            )
            .to("Chris Cochrun <chris@tfcconnection.org>".parse().unwrap())
            .subject(email_subject)
            .singlepart(email)
        {
            crate::email::send_email(m).await
        } else {
            Err(eyre!("Email incorrect"))
        }
    }
}

#[post("/api/mt-parent-form")]
pub async fn mt_parent_form(MultipartForm(mut form): MultipartForm<MtParentForm>) -> HttpResponse {
    match form.store_form().await {
        Ok(_) => info!("Successfully sent form to nextcloud!"),
        Err(e) => error!("There was an erroring sending form to nextcloud: {e}"),
    }
    match form.send_email().await {
        Ok(_) => info!("Successfully sent email"),
        Err(e) => error!("There was an error sending the email: {e}"),
    }
    HttpResponse::Ok().body("thankyou")
}

#[cfg(test)]
mod test {
    use actix_web::test;
    use pretty_assertions::assert_eq;
    use sqlx::Connection;
    use tracing::debug;

    use super::*;

    fn form() -> MtParentForm {
        MtParentForm {
            first_name: Text(String::from("Bilbo")),
            last_name: Text(String::from("Braggins")),
            student_first_name: Text(String::from("Frodo")),
            student_last_name: Text(String::from("Braggins")),
            email: Text(String::from("biblo@hobbits.us")),
            authority: Text(String::from("Uncle")),
            positive: Text(String::from("Nimble and brave")),
            negative: Text(String::from("Small")),
            family: Text(String::from("Such a strutter")),
            previous_trip: Text(String::from("Super")),
            feelings: Text(String::from("Very")),
            extra_info: Text(String::from("Willing to take the ring")),
        }
    }

    #[test]
    async fn test_nc_post() {
        let form = form();
        assert!(!form.first_name.is_empty());
        let res = form.store_form().await;
        match res {
            Ok(_) => assert!(true, "passed storing test"),
            Err(e) => assert!(false, "Failed storing test: {e}"),
        }
    }

    #[test]
    async fn test_email() {
        let mut form = form();
        assert!(!form.first_name.is_empty());
        match form.send_email().await {
            Ok(_) => assert!(true, "passed emailing test"),
            Err(e) => assert!(false, "Failed emailing test: {e}"),
        }
    }
}
